@@ -115,6 +115,11 @@ class SymmetricalCrossSection(BaseModel, frozen=True, arbitrary_types_allowed=Tr
     ) -> SymmetricalCrossSection:
         return super().model_copy(update=update, deep=deep)
 
+    def __eq__(self, o: object) -> bool:
+        if isinstance(o, TCrossSection):
+            return o == self
+        return super().__eq__(o)
+
 
 class DSymmetricalCrossSection(BaseModel):
     """um based CrossSection."""
@@ -226,6 +231,13 @@ class TCrossSection(ABC, Generic[TUnit]):
         self, *, update: Mapping[str, Any] = {"name": None}, deep: bool
     ) -> Self: ...
 
+    def __eq__(self, o: object) -> bool:
+        if isinstance(o, TCrossSection):
+            return self.base == o.base
+        if isinstance(o, SymmetricalCrossSection):
+            return self.base == o
+        return False
+
 
 class CrossSection(TCrossSection[int]):
     @overload
@@ -445,7 +457,7 @@ class DCrossSection(TCrossSection[float]):
         )
 
 
-class TCrossSectionSpec(Generic[TUnit], TypedDict):
+class TCrossSectionSpec(TypedDict, Generic[TUnit]):
     name: NotRequired[str]
     sections: NotRequired[
         list[tuple[kdb.LayerInfo, TUnit] | tuple[kdb.LayerInfo, TUnit, TUnit]]
@@ -550,7 +562,8 @@ class CrossSectionModel(BaseModel):
         if not xs == cross_section:
             raise ValueError(
                 "There is already a cross_section defined with name "
-                f"{cross_section.name}. Cannot overwrite cross_sections."
+                f"{cross_section.name}. Cannot overwrite cross_sections.\n"
+                f"old_xs={xs.model_dump()}\nnew_xs={cross_section.model_dump()}"
             )
         return xs
 
