@@ -7,14 +7,18 @@ the `klayout.db.TilingProcessor` to calculate the regions to fill.
 calculate areas that violate min space violations.
 """
 
+from .difftest import diff, difftest, xor
 from .fill import fill_tiled
 from .simplify import dsimplify, simplify
 from .violations import fix_spacing_minkowski_tiled, fix_spacing_tiled
 
 __all__ = [
+    "diff",
+    "difftest",
     "dsimplify",
     "fill_tiled",
     "fix_spacing_minkowski_tiled",
     "fix_spacing_tiled",
     "simplify",
+    "xor",
 ]
