@@ -1,5 +1,3 @@
-import pytest
-
 import kfactory as kf
 
 
@@ -27,7 +25,3 @@ def test_pack_instances(kcl: kf.KCLayout) -> None:
         c, [ref, ref2, ref3, ref4], max_height=2000, max_width=2000
     )
     assert instance_group.bbox() == kf.kdb.DBox(0, 0, 2000, 2000)
-
-
-if __name__ == "__main__":
-    pytest.main(["-s", __file__])
